activity.setTitle("Created by syed wajeeh ul hassan naqvi ")
require "import"
import "android.widget.*"
import "android.view.*"
import "android.app.*"
import "android.content.*"
import "java.text.SimpleDateFormat"
import "java.util.Date"
import "cjson" -- Using cjson instead of Gson

activity.setContentView(loadlayout({
    LinearLayout,
    orientation="vertical",
    {
        Button,
        id="addButton",
        text="Add New Note",
        layout_width="match_parent",
    },
    {
        ListView,
        id="list",
        layout_width="match_parent",
        layout_height="match_parent",
    },
}))

-- Shared Preferences for persistent storage
local prefs = activity.getSharedPreferences("NotesApp", Context.MODE_PRIVATE)

-- Load saved notes
function loadNotes()
    local savedNotes = prefs.getString("notes", "")
    if savedNotes ~= "" then
        pcall(function()
            notes = cjson.decode(savedNotes)
        end)
    end
    
    if not notes then
        notes = {
            {
                title = "Important Links",
                content = "Google: www.google.com\nYouTube: www.youtube.com",
                date = "2024-01-01"
            },
            {
                title = "Phone Numbers",
                content = "Emergency: 15\nPolice: 15",
                date = "2024-01-01"
            }
        }
    end
end

-- Save notes to storage
function saveNotes()
    local editor = prefs.edit()
    pcall(function()
        local jsonNotes = cjson.encode(notes)
        editor.putString("notes", jsonNotes)
        editor.commit()
    end)
end

-- Function to copy text to clipboard
function copyToClipboard(text)
    activity.getSystemService(Context.CLIPBOARD_SERVICE).setText(text)
    Toast.makeText(activity, "Copied to clipboard!", Toast.LENGTH_SHORT).show()
end

-- Function to share text
function shareText(title, text)
    local intent = Intent(Intent.ACTION_SEND)
    intent.setType("text/plain")
    intent.putExtra(Intent.EXTRA_SUBJECT, title)
    intent.putExtra(Intent.EXTRA_TEXT, text)
    activity.startActivity(Intent.createChooser(intent, "Share Note"))
end

-- Adapter for ListView
function updateList()
    data = {}
    for k,v in ipairs(notes) do
        table.insert(data, v.title.."\n"..v.date)
    end
    adapter = ArrayAdapter(activity,android.R.layout.simple_list_item_1, String(data))
    list.setAdapter(adapter)
    saveNotes() -- Save notes after every update
end

-- Add new note button click
addButton.onClick = function()
    local dialog = Dialog(activity)
    dialog.setContentView(loadlayout({
        LinearLayout,
        orientation="vertical",
        padding="16dp",
        {
            EditText,
            id="title",
            hint="Title",
            layout_width="match_parent",
        },
        {
            EditText,
            id="content",
            hint="Content",
            layout_width="match_parent",
            layout_height="200dp",
            gravity="top",
        },
        {
            Button,
            text="Save",
            layout_width="match_parent",
            onClick=function()
                if title.Text ~= "" and content.Text ~= "" then
                    table.insert(notes, 1, {
                        title=title.Text,
                        content=content.Text,
                        date=os.date("%Y-%m-%d")
                    })
                    updateList()
                    dialog.dismiss()
                    Toast.makeText(activity, "Note saved!", Toast.LENGTH_SHORT).show()
                else
                    Toast.makeText(activity, "Please fill all fields", Toast.LENGTH_SHORT).show()
                end
            end
        }
    }))
    dialog.show()
end

-- Open note on click
list.onItemClick = function(parent, view, position, id)
    local note = notes[position + 1]
    local dialog = Dialog(activity)
    dialog.setContentView(loadlayout({
        LinearLayout,
        orientation="vertical",
        padding="16dp",
        {
            TextView,
            text=note.title,
            textSize="20sp",
            layout_width="match_parent",
        },
        {
            TextView,
            text=note.content,
            layout_width="match_parent",
            layout_marginTop="8dp",
        },
        {
            LinearLayout,
            orientation="horizontal",
            layout_width="match_parent",
            layout_marginTop="16dp",
            {
                Button,
                text="Copy",
                layout_weight="1",
                onClick=function()
                    copyToClipboard(note.content)
                end
            },
            {
                Button,
                text="Share",
                layout_weight="1",
                onClick=function()
                    shareText(note.title, note.content)
                end
            },
            {
                Button,
                text="Close",
                layout_weight="1",
                onClick=function()
                    dialog.dismiss()
                end
            }
        }
    }))
    dialog.show()
end

-- Long press options
list.onItemLongClick = function(parent, view, position, id)
    local note = notes[position + 1]
    local items = {"Edit", "Copy", "Share", "Delete"}
    
    AlertDialog.Builder(activity)
    .setTitle("Options")
    .setItems(items, function(dialog, which)
        if which == 0 then -- Edit
            local editDialog = Dialog(activity)
            editDialog.setContentView(loadlayout({
                LinearLayout,
                orientation="vertical",
                padding="16dp",
                {
                    EditText,
                    id="editTitle",
                    text=note.title,
                    layout_width="match_parent",
                },
                {
                    EditText,
                    id="editContent",
                    text=note.content,
                    layout_width="match_parent",
                    layout_height="200dp",
                    gravity="top",
                },
                {
                    Button,
                    text="Save",
                    layout_width="match_parent",
                    onClick=function()
                        if editTitle.Text ~= "" and editContent.Text ~= "" then
                            notes[position + 1] = {
                                title=editTitle.Text,
                                content=editContent.Text,
                                date=os.date("%Y-%m-%d")
                            }
                            updateList()
                            editDialog.dismiss()
                            Toast.makeText(activity, "Note updated!", Toast.LENGTH_SHORT).show()
                        else
                            Toast.makeText(activity, "Please fill all fields", Toast.LENGTH_SHORT).show()
                        end
                    end
                }
            }))
            editDialog.show()
            
        elseif which == 1 then -- Copy
            copyToClipboard(note.content)
            
        elseif which == 2 then -- Share
            shareText(note.title, note.content)
            
        elseif which == 3 then -- Delete
            AlertDialog.Builder(activity)
            .setTitle("Confirm Delete")
            .setMessage("Are you sure you want to delete this note?")
            .setPositiveButton("Yes", function()
                table.remove(notes, position + 1)
                updateList()
                Toast.makeText(activity, "Note deleted!", Toast.LENGTH_SHORT).show()
            end)
            .setNegativeButton("No", nil)
            .show()
        end
    end)
    .show()
    return true
end

-- Load saved notes and initialize list
loadNotes()
updateList()